@@ -48,6 +48,8 @@ BUILTINPROTO(rebase);
 BUILTINPROTO(rollback);
 BUILTINPROTO(status);
 BUILTINPROTO(db);
+BUILTINPROTO(internals);
+BUILTINPROTO(container);
 
 #undef BUILTINPROTO
 
