@@ -24,7 +24,6 @@
 #include <glib-unix.h>
 #include <json-glib/json-glib.h>
 #include <gio/gunixoutputstream.h>
-#include <rpm/rpmsq.h>
 #include <libhif.h>
 #include <libhif/hif-utils.h>
 #include <stdio.h>
@@ -33,8 +32,8 @@
 
 #include "rpmostree-compose-builtins.h"
 #include "rpmostree-util.h"
+#include "rpmostree-core.h"
 #include "rpmostree-json-parsing.h"
-#include "rpmostree-cleanup.h"
 #include "rpmostree-postprocess.h"
 #include "rpmostree-passwd-util.h"
 #include "rpmostree-libbuiltin.h"
@@ -52,6 +51,7 @@ static char **opt_metadata_strings;
 static char *opt_repo;
 static char **opt_override_pkg_repos;
 static char *opt_touch_if_changed;
+static gboolean opt_dry_run;
 static gboolean opt_print_only;
 
 static GOptionEntry option_entries[] = {
@@ -65,6 +65,7 @@ static GOptionEntry option_entries[] = {
   { "add-override-pkg-repo", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_override_pkg_repos, "Include an additional package repository from DIRECTORY", "DIRECTORY" },
   { "proxy", 0, 0, G_OPTION_ARG_STRING, &opt_proxy, "HTTP proxy", "PROXY" },
   { "touch-if-changed", 0, 0, G_OPTION_ARG_STRING, &opt_touch_if_changed, "Update the modification time on FILE if a new commit was created", "FILE" },
+  { "dry-run", 0, 0, G_OPTION_ARG_NONE, &opt_dry_run, "Just print the transaction and exit", NULL },
   { "print-only", 0, 0, G_OPTION_ARG_NONE, &opt_print_only, "Just expand any includes and print treefile", NULL },
   { NULL }
 };
@@ -89,21 +90,14 @@ typedef struct {
   
   GFile *workdir;
   int workdir_dfd;
+  int cachedir_dfd;
   OstreeRepo *repo;
+  char *ref;
   char *previous_checksum;
 
   GBytes *serialized_treefile;
 } RpmOstreeTreeComposeContext;
 
-static int
-ptrarray_sort_compare_strings (gconstpointer ap,
-                               gconstpointer bp)
-{
-  char **asp = (gpointer)ap;
-  char **bsp = (gpointer)bp;
-  return strcmp (*asp, *bsp);
-}
-
 static gboolean
 compute_checksum_from_treefile_and_goal (RpmOstreeTreeComposeContext   *self,
                                          HyGoal                         goal,
@@ -127,26 +121,7 @@ compute_checksum_from_treefile_and_goal (RpmOstreeTreeComposeContext   *self,
   /* FIXME; we should also hash the post script */
 
   /* Hash in each package */
-  { _cleanup_hypackagelist_ HyPackageList pkglist = NULL;
-    HyPackage pkg;
-    guint i;
-    gs_unref_ptrarray GPtrArray *nevras = g_ptr_array_new_with_free_func (g_free);
-
-    pkglist = hy_goal_list_installs (goal);
-
-    FOR_PACKAGELIST(pkg, pkglist, i)
-      {
-        g_ptr_array_add (nevras, hy_package_get_nevra (pkg));
-      }
-
-    g_ptr_array_sort (nevras, ptrarray_sort_compare_strings);
-    
-    for (i = 0; i < nevras->len; i++)
-      {
-        const char *nevra = nevras->pdata[i];
-        g_checksum_update (checksum, (guint8*)nevra, strlen (nevra));
-      }
-  }
+  rpmostree_hif_add_checksum_goal (checksum, goal);
 
   ret_checksum = g_strdup (g_checksum_get_string (checksum));
 
@@ -166,6 +141,36 @@ on_hifstate_percentage_changed (HifState   *hifstate,
   glnx_console_progress_text_percent (text, percentage);
 }
 
+static gboolean
+set_keyfile_string_array_from_json (GKeyFile    *keyfile,
+                                    const char  *keyfile_group,
+                                    const char  *keyfile_key,
+                                    JsonArray   *a,
+                                    GError     **error)
+{
+  gboolean ret = FALSE;
+  guint len = json_array_get_length (a);
+  guint i;
+  g_autoptr(GPtrArray) instlangs_v = g_ptr_array_new ();
+  
+  for (i = 0; i < len; i++)
+    {
+      const char *elt = _rpmostree_jsonutil_array_require_string_element (a, i, error);
+
+      if (!elt)
+        goto out;
+
+      g_ptr_array_add (instlangs_v, (char*)elt);
+    }
+  
+  g_key_file_set_string_list (keyfile, keyfile_group, keyfile_key,
+                              (const char*const*)instlangs_v->pdata, instlangs_v->len);
+
+  ret = TRUE;
+ out:
+  return ret;
+}
+
 static gboolean
 install_packages_in_root (RpmOstreeTreeComposeContext  *self,
                           JsonObject      *treedata,
@@ -178,146 +183,62 @@ install_packages_in_root (RpmOstreeTreeComposeContext  *self,
 {
   gboolean ret = FALSE;
   guint progress_sigid;
-  char **strviter;
   GFile *contextdir = self->treefile_context_dirs->pdata[0];
-  gs_unref_object HifContext *hifctx = NULL;
-  gs_free char *cachedir = g_build_filename (gs_file_get_path_cached (self->workdir),
-                                             "cache",
-                                             NULL);
-  gs_free char *solvdir = g_build_filename (gs_file_get_path_cached (self->workdir),
-                                            "solv",
-                                            NULL);
-  gs_free char *lockdir = g_build_filename (gs_file_get_path_cached (self->workdir),
-                                            "lock",
-                                            NULL);
+  g_autoptr(RpmOstreeInstall) hifinstall = { 0, };
+  g_autoptr(RpmOstreeContext) ctx = NULL;
+  HifContext *hifctx;
   gs_free char *ret_new_inputhash = NULL;
+  g_autoptr(GKeyFile) treespec = g_key_file_new ();
+  JsonArray *enable_repos = NULL;
 
-  /* We can be control-c'd at any time */
-#if BUILDOPT_HAVE_RPMSQ_SET_INTERRUPT_SAFETY
-  rpmsqSetInterruptSafety (FALSE);
+  /* TODO - uncomment this once we have SELinux working */
+#if 0
+  g_autofree char *cache_repo_pathstr = glnx_fdrel_abspath (self->cachedir_dfd, "repo");
+  g_autoptr(GFile) cache_repo_path = g_file_new_for_path (cache_repo_pathstr);
+  glnx_unref_object OstreeRepo *ostreerepo = ostree_repo_new (cache_repo_path);
+
+  if (!g_file_test (cache_repo_pathstr, G_FILE_TEST_EXISTS))
+    {
+      if (!ostree_repo_create (ostreerepo, OSTREE_REPO_MODE_BARE_USER, cancellable, error))
+        goto out;
+    }
 #endif
 
-  /* Apparently there's only one process-global macro context;
-   * realistically, we're going to have to refactor all of the RPM
-   * stuff to a subprocess.
-   */
-  hifctx = hif_context_new ();
-  hif_context_set_http_proxy (hifctx, opt_proxy ? opt_proxy : g_getenv ("http_proxy"));
-
-  hif_context_set_install_root (hifctx, gs_file_get_path_cached (yumroot));
-
-  hif_context_set_cache_dir (hifctx, cachedir);
-  hif_context_set_cache_age (hifctx, G_MAXUINT);
-  hif_context_set_solv_dir (hifctx, solvdir);
-  hif_context_set_lock_dir (hifctx, lockdir);
-  hif_context_set_check_disk_space (hifctx, FALSE);
-  hif_context_set_check_transaction (hifctx, FALSE);
-
-  hif_context_set_yumdb_enabled (hifctx, FALSE);
+  ctx = rpmostree_context_new_unprivileged (self->cachedir_dfd, cancellable, error);
+  if (!ctx)
+    goto out;
+  hifctx = rpmostree_context_get_hif (ctx);
+  if (opt_proxy)
+    hif_context_set_http_proxy (hifctx, opt_proxy);
 
   hif_context_set_repo_dir (hifctx, gs_file_get_path_cached (contextdir));
 
-  { JsonNode *install_langs_n =
-      json_object_get_member (treedata, "install-langs");
+  g_key_file_set_string (treespec, "tree", "ref", self->ref);
+  g_key_file_set_string_list (treespec, "tree", "packages", (const char *const*)packages, g_strv_length (packages));
 
-    if (install_langs_n != NULL)
-      {
-        JsonArray *instlangs_a = json_node_get_array (install_langs_n);
-        guint len = json_array_get_length (instlangs_a);
-        guint i;
-        GString *opt = g_string_new ("");
-
-        for (i = 0; i < len; i++)
-          {
-            g_string_append (opt, json_array_get_string_element (instlangs_a, i));
-            if (i < len - 1)
-              g_string_append_c (opt, ':');
-          }
-
-        hif_context_set_rpm_macro (hifctx, "_install_langs", opt->str);
-        g_string_free (opt, TRUE);
-      }
-  }
-
-  if (!hif_context_setup (hifctx, cancellable, error))
-    goto out;
-
-  /* Forcibly override rpm/librepo SIGINT handlers.  We always operate
-   * in a fully idempotent/atomic mode, and can be killed at any time.
-   */
-  signal (SIGINT, SIG_DFL);
-  signal (SIGTERM, SIG_DFL);
+  /* Some awful code to translate between JSON and GKeyFile */
+  if (json_object_has_member (treedata, "install-langs"))
+    {
+      JsonArray *a = json_object_get_array_member (treedata, "install-langs");
+      if (!set_keyfile_string_array_from_json (treespec, "tree", "install-langs", a, error))
+        goto out;
+    }
 
   /* Bind the json \"repos\" member to the hif state, which looks at the
    * enabled= member of the repos file.  By default we forcibly enable
    * only repos which are specified, ignoring the enabled= flag.
    */
-  {
-    GPtrArray *sources;
-    JsonArray *enable_repos = NULL;
-    gs_unref_hashtable GHashTable *repos_to_enable =
-      g_hash_table_new (g_str_hash, g_str_equal);
-    guint i;
-    guint n;
+  if (!json_object_has_member (treedata, "repos"))
+    {
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                           "Treefile is missing required \"repos\" member");
+      goto out;
+    }
 
-    sources = hif_context_get_sources (hifctx);
+  enable_repos = json_object_get_array_member (treedata, "repos");
 
-    if (!json_object_has_member (treedata, "repos"))
-      {
-        g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                             "Treefile is missing required \"repos\" member");
-        goto out;
-      }
-
-    enable_repos = json_object_get_array_member (treedata, "repos");
-    n = json_array_get_length (enable_repos);
-
-    for (i = 0; i < n; i++)
-      {
-        const char *reponame = _rpmostree_jsonutil_array_require_string_element (enable_repos, i, error);
-        if (!reponame)
-          goto out;
-        g_hash_table_add (repos_to_enable, (char*)reponame);
-      }
-
-    for (i = 0; i < sources->len; i++)
-      {
-        HifSource *src = g_ptr_array_index (sources, i);
-        const char *id = hif_source_get_id (src);
-
-        /* Note side effect here */
-        if (g_hash_table_remove (repos_to_enable, id))
-          {
-            hif_source_set_enabled (src, HIF_SOURCE_ENABLED_PACKAGES);
-#ifdef HAVE_HIF_SOURCE_SET_REQUIRED
-            hif_source_set_required (src, TRUE);
-#endif
-          }
-        else
-          hif_source_set_enabled (src, HIF_SOURCE_ENABLED_NONE);
-      }
-    
-    /* Did we have any leftovers? */
-    if (g_hash_table_size (repos_to_enable) > 0)
-      {
-        GString *repos_str = g_string_new ("");
-        GHashTableIter hiter;
-        gpointer key, value;
-        
-        g_hash_table_iter_init (&hiter, repos_to_enable);
-        while (g_hash_table_iter_next (&hiter, &key, &value))
-          {
-            if (repos_str->len > 0)
-              g_string_append (repos_str, ", ");
-            g_string_append (repos_str, (char*) key);
-          }
-
-        g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                     "Repositories specified not found: %s", repos_str->str);
-        g_string_free (repos_str, TRUE);
-        goto out;
-      }
-  }
+  if (!set_keyfile_string_array_from_json (treespec, "tree", "repos", enable_repos, error))
+    goto out;
 
   { gboolean docs = TRUE;
 
@@ -328,50 +249,26 @@ install_packages_in_root (RpmOstreeTreeComposeContext  *self,
       goto out;
 
     if (!docs)
-      hif_transaction_set_flags (hif_context_get_transaction (hifctx),
-                                 HIF_TRANSACTION_FLAG_NODOCS);
+      g_key_file_set_boolean (treespec, "tree", "documentation", FALSE);
+  }
+
+  { g_autoptr(GError) tmp_error = NULL;
+    g_autoptr(RpmOstreeTreespec) treespec_value = rpmostree_treespec_new_from_keyfile (treespec, &tmp_error);
+    g_assert_no_error (tmp_error);
+    
+    if (!rpmostree_context_setup (ctx, gs_file_get_path_cached (yumroot), treespec_value,
+                                  cancellable, error))
+      goto out;
   }
 
   /* --- Downloading metadata --- */
-  { g_auto(GLnxConsoleRef) console = { 0, };
-    gs_unref_object HifState *hifstate = hif_state_new ();
+  if (!rpmostree_context_download_metadata (ctx, cancellable, error))
+    goto out;
 
-    progress_sigid = g_signal_connect (hifstate, "percentage-changed",
-                                     G_CALLBACK (on_hifstate_percentage_changed), 
-                                     "Downloading metadata:");
-
-    glnx_console_lock (&console);
-
-    if (!hif_context_setup_sack (hifctx, hifstate, error))
-      goto out;
-
-    g_signal_handler_disconnect (hifstate, progress_sigid);
-  }
-
-  for (strviter = packages; strviter && *strviter; strviter++)
-    {
-      if (!hif_context_install (hifctx, *strviter, error))
-        goto out;
-    }
-
-  /* --- Resolving dependencies --- */
-  { g_auto(GLnxConsoleRef) console = { 0, };
-    gs_unref_object HifState *hifstate = hif_state_new ();
-
-    progress_sigid = g_signal_connect (hifstate, "percentage-changed",
-                                     G_CALLBACK (on_hifstate_percentage_changed), 
-                                     "Resolving dependencies:");
-
-    glnx_console_lock (&console);
-
-    if (!hif_transaction_depsolve (hif_context_get_transaction (hifctx),
-                                   hif_context_get_goal (hifctx),
-                                   hifstate, error))
-      goto out;
-
-    g_signal_handler_disconnect (hifstate, progress_sigid);
-  }
+  if (!rpmostree_context_prepare_install (ctx, &hifinstall, cancellable, error))
+    goto out;
 
+  /* FIXME - just do a depsolve here before we compute download requirements */
   if (!compute_checksum_from_treefile_and_goal (self, hif_context_get_goal (hifctx),
                                                 &ret_new_inputhash, error))
     goto out;
@@ -402,24 +299,16 @@ install_packages_in_root (RpmOstreeTreeComposeContext  *self,
         g_print ("Previous commit found, but without rpmostree.inputhash metadata key\n");
     }
 
-  rpmostree_print_transaction (hifctx);
+  if (opt_dry_run)
+    {
+      ret = TRUE;
+      goto out;
+    }
 
   /* --- Downloading packages --- */
-  { g_auto(GLnxConsoleRef) console = { 0, };
-    gs_unref_object HifState *hifstate = hif_state_new ();
-
-    progress_sigid = g_signal_connect (hifstate, "percentage-changed",
-                                     G_CALLBACK (on_hifstate_percentage_changed), 
-                                     "Downloading packages:");
-
-    glnx_console_lock (&console);
-
-    if (!hif_transaction_download (hif_context_get_transaction (hifctx), hifstate, error))
-      goto out;
-
-    g_signal_handler_disconnect (hifstate, progress_sigid);
-  }
-
+  if (!rpmostree_context_download_rpms (ctx, -1, hifinstall, cancellable, error))
+    goto out;
+  
   { g_auto(GLnxConsoleRef) console = { 0, };
     gs_unref_object HifState *hifstate = hif_state_new ();
 
@@ -624,14 +513,12 @@ rpmostree_compose_builtin_tree (int             argc,
   int exit_status = EXIT_FAILURE;
   GError *temp_error = NULL;
   GOptionContext *context = g_option_context_new ("TREEFILE - Run yum and commit the result to an OSTree repository");
-  const char *ref;
   RpmOstreeTreeComposeContext selfdata = { NULL, };
   RpmOstreeTreeComposeContext *self = &selfdata;
   JsonNode *treefile_rootval = NULL;
   JsonObject *treefile = NULL;
   gs_free char *cachekey = NULL;
   gs_free char *new_inputhash = NULL;
-  gs_unref_object GFile *cachedir = NULL;
   gs_unref_object GFile *previous_root = NULL;
   gs_free char *previous_checksum = NULL;
   gs_unref_object GFile *yumroot = NULL;
@@ -715,9 +602,20 @@ rpmostree_compose_builtin_tree (int             argc,
 
   if (opt_cachedir)
     {
-      cachedir = g_file_new_for_path (opt_cachedir);
-      if (!gs_file_ensure_directory (cachedir, FALSE, cancellable, error))
-        goto out;
+      if (!glnx_opendirat (AT_FDCWD, opt_cachedir, TRUE, &self->cachedir_dfd, error))
+        {
+          g_prefix_error (error, "Opening cachedir '%s': ", opt_cachedir);
+          goto out;
+        }
+    }
+  else
+    {
+      self->cachedir_dfd = fcntl (self->workdir_dfd, F_DUPFD_CLOEXEC, 3);
+      if (self->cachedir_dfd < 0)
+        {
+          glnx_set_error_from_errno (error);
+          goto out;
+        }
     }
 
   if (opt_metadata_strings)
@@ -765,17 +663,17 @@ rpmostree_compose_builtin_tree (int             argc,
       goto out;
     }
 
-  ref = _rpmostree_jsonutil_object_require_string_member (treefile, "ref", error);
-  if (!ref)
+  self->ref = g_strdup (_rpmostree_jsonutil_object_require_string_member (treefile, "ref", error));
+  if (!self->ref)
     goto out;
 
-  if (!ostree_repo_read_commit (repo, ref, &previous_root, &previous_checksum,
+  if (!ostree_repo_read_commit (repo, self->ref, &previous_root, &previous_checksum,
                                 cancellable, &temp_error))
     {
       if (g_error_matches (temp_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
         { 
           g_clear_error (&temp_error);
-          g_print ("No previous commit for %s\n", ref);
+          g_print ("No previous commit for %s\n", self->ref);
         }
       else
         {
@@ -832,8 +730,7 @@ rpmostree_compose_builtin_tree (int             argc,
     goto out;
   g_ptr_array_add (packages, NULL);
 
-  {
-    gs_unref_object JsonGenerator *generator = json_generator_new ();
+  { gs_unref_object JsonGenerator *generator = json_generator_new ();
     char *treefile_buf = NULL;
     gsize len;
 
@@ -880,6 +777,12 @@ rpmostree_compose_builtin_tree (int             argc,
         exit_status = EXIT_SUCCESS;
         goto out;
       }
+    else if (opt_dry_run)
+      {
+        g_print ("--dry-run complete, exiting\n");
+        exit_status = EXIT_SUCCESS;
+        goto out;
+      }
   }
 
   if (g_strcmp0 (g_getenv ("RPM_OSTREE_BREAK"), "post-yum") == 0)
@@ -921,7 +824,7 @@ rpmostree_compose_builtin_tree (int             argc,
                                                                  error))
       goto out;
 
-    if (!rpmostree_commit (yumroot, repo, ref, metadata, gpgkey, selinux,
+    if (!rpmostree_commit (yumroot, repo, self->ref, metadata, gpgkey, selinux,
                            cancellable, error))
       goto out;
   }
